use crate::config::Config;
use crate::player;
use crate::track::Track;
use anyhow::{Context, Result};
use mpris::PlaybackStatus;
use std::thread;
use std::time::{Duration, Instant};
use rustfm_scrobble_proxy::{Scrobble, Scrobbler};
use tracing::info;

const INTERVAL: Duration = Duration::from_millis(1000);

// see https://www.last.fm/api/scrobbling#scrobble-requests
// do not scrobble songs over 30 seconds, scrobble them if we are 4 minutes through,
// or halfway through (whichever occurs first)
const MIN_SONG_LENGTH: Duration = Duration::from_secs(30);
const MIN_SCROBBLE_POSITION: Duration = Duration::from_mins(4);
fn do_we_scrobble(position: Duration, length: Option<Duration>) -> bool {
    match length {
        Some(length) if length <= Duration::from_secs(30) => false,
        Some(length) => position >= MIN_SCROBBLE_POSITION || position >= length / 2,
        // we can't exactly appeal the spec here... but we don't also want only songs over 30 seconds to work
        // because of this, it is a much safer option to use the min song length
        None => position >= MIN_SONG_LENGTH,
    }
}

pub fn start(config: &Config, scrobbler: Scrobbler) -> Result<()> {
    info!(" looking for an active mpris player");

    let mut player = player::wait_for_player(config)?;

    info!(" found an active player: {}", player.identity());

    let mut previously_logged_track = Track::default();

    let mut timer = Instant::now();
    let mut current_play_time = Duration::from_secs(0);
    let mut scrobbled_current_song = false;

    loop {
        if !player::is_active(&player) {
            info!(" player {} stopped, searching for an active mpris player", player.identity());

            player = player::wait_for_player(config)
                .context("failed to find player")?;

            info!(" found an active player: {}", player.identity());
        }

        let playback_status = player
            .get_playback_status()
            .context("failed to retrieve playback status")?;

        match playback_status {
            PlaybackStatus::Playing => {}
            _ => {
                thread::sleep(INTERVAL);
                continue;
            }
        }

        let metadata = player
            .get_metadata()
            .context("failed to get metadata")?;

        let current_track = Track::from_metadata(&metadata);

        let length = metadata
            .length()
            .and_then(|length| if length.is_zero() { None } else { Some(length) });

        if current_track == previously_logged_track {
            if !scrobbled_current_song && do_we_scrobble(current_play_time, length) {
                let scrobble = Scrobble::new(current_track.artist(), current_track.title(), current_track.album());
                scrobbler.scrobble(&scrobble)
                    .context("failed to scrobble")?;

                info!(" scrobble submitted sucessfully");

                scrobbled_current_song = true
            } else if length
                .map(|length| current_play_time >= length)
                .unwrap_or(false) 
            {
                current_play_time = Duration::from_secs(0);
                scrobbled_current_song = false;
            }

            current_play_time += timer.elapsed();
            timer = Instant::now();
        } else {
            previously_logged_track.clone_from(&current_track);

            timer = Instant::now();
            current_play_time = Duration::from_secs(0);
            scrobbled_current_song = false;

            info!(
                " now playing {} by {}{}",
                current_track.title(),
                current_track.artist(),
                match current_track.album() {
                    Some(album) => format!(" on {}", album),
                    None => String::new(),
                }
            );

            let scrobble = Scrobble::new(current_track.artist(), current_track.title(), current_track.album());
            scrobbler.now_playing(&scrobble)
                .context("failed to update now playing status")?;

            info!(" now playing status updated successfully");
        }

        thread::sleep(INTERVAL);
    }
}
