use mpris::Metadata;

#[derive(Debug, Default, PartialEq)]
pub struct Track {
    artist: String,
    title: String,
    album: Option<String>,
}

impl Track {
    pub fn artist(&self) -> &str {
        &self.artist
    }

    pub fn title(&self) -> &str {
        &self.title
    }

    pub fn album(&self) -> Option<&str> {
        self.album.as_deref()
    }

    pub fn new(artist: &str, title: &str, album: Option<&str>) -> Self {
        Self {
            artist: artist.to_owned(),
            title: title.to_owned(),
            album: album.and_then(|album| {
                if !album.is_empty() {
                    Some(album.to_owned())
                } else {
                    None
                }
            }),
        }
    }

    pub fn clear(&mut self) {
        self.artist.clear();
        self.title.clear();
        self.album.take();
    }

    pub fn clone_from(&mut self, other: &Self) {
        self.artist.clone_from(&other.artist);
        self.title.clone_from(&other.title);
        self.album.clone_from(&other.album);
    }

    pub fn from_metadata(metadata: &Metadata) -> Self {
        // these unknown checks are required or else scrobbling will return a bad request
        // last.fm can handle unknown artist kinda well!
        let artist = metadata
            .artists()
            .as_ref()
            .and_then(|artists| artists.first().copied())
            .unwrap_or("Unknown Artist")
            .to_owned();

        // unknown song on the other hand.. well, it just scrobbles the song "Unknown Song"
        // this is probably better than getting a bad request and the program crashing :3
        let title = metadata.title().unwrap_or("Unknown Song").to_owned();

        let album = metadata.album_name().and_then(|album| {
            if !album.is_empty() {
                Some(album.to_owned())
            } else {
                None
            }
        });

        Self {
            artist,
            title,
            album,
        }
    }
}